@@ -3,7 +3,7 @@
  * Zend Framework (http://framework.zend.com/)
  *
  * @link      http://github.com/zendframework/zf2 for the canonical source repository
- * @copyright Copyright (c) 2005-2013 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license   http://framework.zend.com/license/new-bsd New BSD License
  */
 
@@ -18,7 +18,11 @@ use Zend\Stdlib\InitializableInterface;
  * @todo       How should we deal with required input when data is missing?
  *             should a message be returned? if so, what message?
  */
-class BaseInputFilter implements InputFilterInterface, UnknownInputsCapableInterface, InitializableInterface
+class BaseInputFilter implements
+    InputFilterInterface,
+    UnknownInputsCapableInterface,
+    InitializableInterface,
+    ReplaceableInputInterface
 {
     protected $data;
     protected $inputs = array();
@@ -84,6 +88,38 @@ class BaseInputFilter implements InputFilterInterface, UnknownInputsCapableInter
         return $this;
     }
 
+    /**
+     * Replace a named input
+     *
+     * @param  InputInterface|InputFilterInterface $input
+     * @param  string                              $name Name of the input to replace
+     * @throws Exception\InvalidArgumentException
+     * @return self
+     */
+    public function replace($input, $name)
+    {
+        if (!$input instanceof InputInterface && !$input instanceof InputFilterInterface) {
+            throw new Exception\InvalidArgumentException(sprintf(
+                '%s expects an instance of %s or %s as its first argument; received "%s"',
+                __METHOD__,
+                'Zend\InputFilter\InputInterface',
+                'Zend\InputFilter\InputFilterInterface',
+                (is_object($input) ? get_class($input) : gettype($input))
+            ));
+        }
+
+        if (!array_key_exists($name, $this->inputs)) {
+            throw new Exception\InvalidArgumentException(sprintf(
+                '%s: no input found matching "%s"',
+                __METHOD__,
+                $name
+            ));
+        }
+
+        $this->inputs[$name] = $input;
+        return $this;
+    }
+
     /**
      * Retrieve a named input
      *
@@ -209,7 +245,7 @@ class BaseInputFilter implements InputFilterInterface, UnknownInputsCapableInter
                 && $input->isRequired()
                 && $input->allowEmpty()
             ) {
-                if(!($input instanceOf EmptyContextInterface && $input->continueIfEmpty())) {
+                if (!($input instanceOf EmptyContextInterface && $input->continueIfEmpty())) {
                     $this->validInputs[$name] = $input;
                     continue;
                 }
@@ -339,6 +375,11 @@ class BaseInputFilter implements InputFilterInterface, UnknownInputsCapableInter
     {
         if ($name === self::VALIDATE_ALL) {
             $this->validationGroup = null;
+            foreach($this->getInputs() as $input) {
+                if ($input instanceof InputFilterInterface) {
+                    $input->setValidationGroup(self::VALIDATE_ALL);
+                }
+            }
             return $this;
         }
 
@@ -350,6 +391,14 @@ class BaseInputFilter implements InputFilterInterface, UnknownInputsCapableInter
                 } else {
                     $inputs[] = $key;
 
+                    if (!$this->inputs[$key] instanceof InputFilterInterface) {
+                        throw new Exception\InvalidArgumentException(
+                            sprintf(
+                                'Input "%s" must implement InputFilterInterface',
+                                $key
+                            )
+                        );
+                    }
                     // Recursively populate validation groups for sub input filters
                     $this->inputs[$key]->setValidationGroup($value);
                 }
@@ -532,6 +581,15 @@ class BaseInputFilter implements InputFilterInterface, UnknownInputsCapableInter
                 // No value; clear value in this input
                 if ($input instanceof InputFilterInterface) {
                     $input->setData(array());
+                    if ($input instanceof CollectionInputFilter) {
+                        $input->clearValues();
+                        $input->clearRawValues();
+                    }
+                    continue;
+                }
+
+                if ($input instanceof ArrayInput) {
+                    $input->setValue(array());
                     continue;
                 }
 
