@@ -3,7 +3,7 @@
  * Zend Framework (http://framework.zend.com/)
  *
  * @link      http://github.com/zendframework/zf2 for the canonical source repository
- * @copyright Copyright (c) 2005-2013 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license   http://framework.zend.com/license/new-bsd New BSD License
  */
 
@@ -115,6 +115,13 @@ abstract class AbstractPage extends AbstractContainer
      */
     protected $permission;
 
+    /**
+     * Text domain for Translator
+     *
+     * @var string
+     */
+    protected $textDomain;
+
     /**
      * Whether this page should be considered active
      *
@@ -143,6 +150,13 @@ abstract class AbstractPage extends AbstractContainer
      */
     protected $properties = array();
 
+    /**
+     * Static factories list for factory pages
+     *
+     * @var array
+     */
+    protected static $factories = array();
+
     // Initialization:
 
     /**
@@ -220,6 +234,14 @@ abstract class AbstractPage extends AbstractContainer
             }
         }
 
+        if (static::$factories) {
+            foreach (static::$factories as $factoryCallBack) {
+                if (($page = call_user_func($factoryCallBack, $options))) {
+                    return $page;
+                }
+            }
+        }
+
         $hasUri = isset($options['uri']);
         $hasMvc = isset($options['action']) || isset($options['controller'])
                 || isset($options['route']);
@@ -235,6 +257,16 @@ abstract class AbstractPage extends AbstractContainer
         }
     }
 
+    /**
+     * Add static factory for self::factory function
+     *
+     * @param type $callback Any callable variable
+     */
+    public static function addFactory($callback)
+    {
+        static::$factories[] = $callback;
+    }
+
     /**
      * Page constructor
      *
@@ -732,6 +764,33 @@ abstract class AbstractPage extends AbstractContainer
         return $this->permission;
     }
 
+    /**
+     * Sets text domain for translation
+     *
+     * @param  string|null $textDomain  [optional] text domain to associate
+     *                                  with this page. Default is null, which
+     *                                  sets no text domain.
+     *
+     * @return AbstractPage fluent interface, returns self
+     */
+    public function setTextDomain($textDomain = null)
+    {
+        if (null !== $textDomain) {
+            $this->textDomain = $textDomain;
+        }
+        return $this;
+    }
+
+    /**
+     * Returns text domain for translation
+     *
+     * @return mixed|null  text domain or null
+     */
+    public function getTextDomain()
+    {
+        return $this->textDomain;
+    }
+
     /**
      * Sets whether page should be considered active or not
      *
