@@ -3,7 +3,7 @@
  * Zend Framework (http://framework.zend.com/)
  *
  * @link      http://github.com/zendframework/zf2 for the canonical source repository
- * @copyright Copyright (c) 2005-2013 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license   http://framework.zend.com/license/new-bsd New BSD License
  */
 namespace Zend\Test\PHPUnit\Controller;
@@ -181,7 +181,7 @@ abstract class AbstractControllerTestCase extends PHPUnit_Framework_TestCase
      */
     public function getResponse()
     {
-        return $this->getApplication()->getResponse();
+        return $this->getApplication()->getMvcEvent()->getResponse();
     }
 
     /**
@@ -242,7 +242,7 @@ abstract class AbstractControllerTestCase extends PHPUnit_Framework_TestCase
     }
 
     /**
-     * Dispatch the MVC with an URL
+     * Dispatch the MVC with a URL
      * Accept a HTTP (simulate a customer action) or console route.
      *
      * The URL provided set the request URI in the request object.
@@ -252,7 +252,7 @@ abstract class AbstractControllerTestCase extends PHPUnit_Framework_TestCase
      * @param  array|null $params
      * @throws \Exception
      */
-    public function dispatch($url, $method = null, $params = array())
+    public function dispatch($url, $method = null, $params = array(), $isXmlHttpRequest = false)
     {
         if ( !isset($method) &&
              $this->getRequest() instanceof HttpRequest &&
@@ -263,6 +263,11 @@ abstract class AbstractControllerTestCase extends PHPUnit_Framework_TestCase
             $method = HttpRequest::METHOD_GET;
         }
 
+        if ($isXmlHttpRequest) {
+            $headers = $this->getRequest()->getHeaders();
+            $headers->addHeaderLine('X_REQUESTED_WITH', 'XMLHttpRequest');
+        }
+
         $this->url($url, $method, $params);
         $this->getApplication()->run();
 
@@ -281,16 +286,19 @@ abstract class AbstractControllerTestCase extends PHPUnit_Framework_TestCase
      *
      * @return AbstractControllerTestCase
      */
-    public function reset()
+    public function reset($keepPersistence = false)
     {
         // force to re-create all components
         $this->application = null;
 
         // reset server datas
-        $_SESSION = array();
+        if (!$keepPersistence) {
+            $_SESSION = array();
+            $_COOKIE  = array();
+        }
+
         $_GET     = array();
         $_POST    = array();
-        $_COOKIE  = array();
 
         // reset singleton
         StaticEventManager::resetInstance();
@@ -462,7 +470,7 @@ abstract class AbstractControllerTestCase extends PHPUnit_Framework_TestCase
     {
         $routeMatch           = $this->getApplication()->getMvcEvent()->getRouteMatch();
         $controllerIdentifier = $routeMatch->getParam('controller');
-        $controllerManager    = $this->getApplicationServiceLocator()->get('ControllerLoader');
+        $controllerManager    = $this->getApplicationServiceLocator()->get('ControllerManager');
         $controllerClass      = $controllerManager->get($controllerIdentifier);
         return get_class($controllerClass);
     }
@@ -665,4 +673,46 @@ abstract class AbstractControllerTestCase extends PHPUnit_Framework_TestCase
         }
         $this->assertNotEquals($route, $match);
     }
+
+    /**
+     * Assert template name
+     * Assert that a template was used somewhere in the view model tree
+     *
+     * @param  string $templateName
+     */
+    public function assertTemplateName($templateName)
+    {
+        $viewModel = $this->getApplication()->getMvcEvent()->getViewModel();
+        $this->assertTrue($this->searchTemplates($viewModel, $templateName));
+    }
+
+    /**
+     * Assert not template name
+     * Assert that a template was not used somewhere in the view model tree
+     *
+     * @param  string $templateName
+     */
+    public function assertNotTemplateName($templateName)
+    {
+        $viewModel = $this->getApplication()->getMvcEvent()->getViewModel();
+        $this->assertFalse($this->searchTemplates($viewModel, $templateName));
+    }
+
+    /**
+     * Recursively search a view model and it's children for the given templateName
+     *
+     * @param ViewModel $viewModel
+     * @param string $templateName
+     * @return boolean
+     */
+    protected function searchTemplates($viewModel, $templateName)
+    {
+        if ($viewModel->getTemplate($templateName) == $templateName){
+            return true;
+        }
+        foreach ($viewModel->getChildren() as $child){
+            return $this->searchTemplates($child, $templateName);
+        }
+        return false;
+    }
 }
