@@ -3,13 +3,13 @@
  * Zend Framework (http://framework.zend.com/)
  *
  * @link      http://github.com/zendframework/zf2 for the canonical source repository
- * @copyright Copyright (c) 2005-2013 Zend Technologies USA Inc. (http://www.zend.com)
+ * @copyright Copyright (c) 2005-2014 Zend Technologies USA Inc. (http://www.zend.com)
  * @license   http://framework.zend.com/license/new-bsd New BSD License
  */
 namespace Zend\Test\PHPUnit\Controller;
 
 use PHPUnit_Framework_ExpectationFailedException;
-use Zend\Dom;
+use Zend\Dom\Document;
 
 abstract class AbstractHttpControllerTestCase extends AbstractControllerTestCase
 {
@@ -39,6 +39,16 @@ abstract class AbstractHttpControllerTestCase extends AbstractControllerTestCase
         return $responseHeader;
     }
 
+    /**
+     * Assert response has the given reason phrase
+     *
+     * @param string $phrase
+     */
+    public function assertResponseReasonPhrase($phrase)
+    {
+        $this->assertEquals($phrase, $this->getResponse()->getReasonPhrase());
+    }
+
     /**
      * Assert response header exists
      *
@@ -88,15 +98,30 @@ abstract class AbstractHttpControllerTestCase extends AbstractControllerTestCase
                 $header
             ));
         }
-        if ($match != $responseHeader->getFieldValue()) {
-            throw new PHPUnit_Framework_ExpectationFailedException(sprintf(
+
+        if (!$responseHeader instanceof \ArrayIterator) {
+            $responseHeader = array($responseHeader);
+        }
+
+        $headerMatched = false;
+
+        foreach ($responseHeader as $currentHeader) {
+            if ($match == $currentHeader->getFieldValue()) {
+                $headerMatched = true;
+                break;
+            }
+        }
+
+        if (!$headerMatched) {
+            throw new \PHPUnit_Framework_ExpectationFailedException(sprintf(
                 'Failed asserting response header "%s" exists and contains "%s", actual content is "%s"',
                 $header,
                 $match,
-                $responseHeader->getFieldValue()
+                $currentHeader->getFieldValue()
             ));
         }
-        $this->assertEquals($match, $responseHeader->getFieldValue());
+
+        $this->assertEquals($match, $currentHeader->getFieldValue());
     }
 
     /**
@@ -114,14 +139,22 @@ abstract class AbstractHttpControllerTestCase extends AbstractControllerTestCase
                 $header
             ));
         }
-        if ($match == $responseHeader->getFieldValue()) {
-            throw new PHPUnit_Framework_ExpectationFailedException(sprintf(
-                'Failed asserting response header "%s" DOES NOT CONTAIN "%s"',
-                $header,
-                $match
-            ));
+
+        if (!$responseHeader instanceof \ArrayIterator) {
+            $responseHeader = array($responseHeader);
         }
-        $this->assertNotEquals($match, $responseHeader->getFieldValue());
+
+        foreach ($responseHeader as $currentHeader) {
+            if ($match == $currentHeader->getFieldValue()) {
+                throw new PHPUnit_Framework_ExpectationFailedException(sprintf(
+                    'Failed asserting response header "%s" DOES NOT CONTAIN "%s"',
+                    $header,
+                    $match
+                ));
+            }
+        }
+
+        $this->assertNotEquals($match, $currentHeader->getFieldValue());
     }
 
     /**
@@ -139,15 +172,31 @@ abstract class AbstractHttpControllerTestCase extends AbstractControllerTestCase
                 $header
             ));
         }
-        if (!preg_match($pattern, $responseHeader->getFieldValue())) {
+
+        if (!$responseHeader instanceof \ArrayIterator) {
+            $responseHeader = array($responseHeader);
+        }
+
+        $headerMatched = false;
+
+        foreach ($responseHeader as $currentHeader) {
+            $headerMatched = (bool) preg_match($pattern, $currentHeader->getFieldValue());
+
+            if ($headerMatched) {
+                break;
+            }
+        }
+
+        if (!$headerMatched) {
             throw new PHPUnit_Framework_ExpectationFailedException(sprintf(
                 'Failed asserting response header "%s" exists and matches regex "%s", actual content is "%s"',
                 $header,
                 $pattern,
-                $responseHeader->getFieldValue()
+                $currentHeader->getFieldValue()
             ));
         }
-        $this->assertTrue((bool) preg_match($pattern, $responseHeader->getFieldValue()));
+
+        $this->assertTrue($headerMatched);
     }
 
     /**
@@ -165,14 +214,26 @@ abstract class AbstractHttpControllerTestCase extends AbstractControllerTestCase
                 $header
             ));
         }
-        if (preg_match($pattern, $responseHeader->getFieldValue())) {
-            throw new PHPUnit_Framework_ExpectationFailedException(sprintf(
-                'Failed asserting response header "%s" DOES NOT MATCH regex "%s"',
-                $header,
-                $pattern
-            ));
+
+        if (!$responseHeader instanceof \ArrayIterator) {
+            $responseHeader = array($responseHeader);
         }
-        $this->assertFalse((bool) preg_match($pattern, $responseHeader->getFieldValue()));
+
+        $headerMatched = false;
+
+        foreach ($responseHeader as $currentHeader) {
+            $headerMatched = (bool) preg_match($pattern, $currentHeader->getFieldValue());
+
+            if ($headerMatched) {
+                throw new PHPUnit_Framework_ExpectationFailedException(sprintf(
+                    'Failed asserting response header "%s" DOES NOT MATCH regex "%s"',
+                    $header,
+                    $pattern
+                ));
+            }
+        }
+
+        $this->assertFalse($headerMatched);
     }
 
     /**
@@ -309,17 +370,20 @@ abstract class AbstractHttpControllerTestCase extends AbstractControllerTestCase
      *
      * @param  string $path
      * @param  bool $useXpath
-     * @return array
+     * @return Document\NodeList
      */
     private function query($path, $useXpath = false)
     {
         $response = $this->getResponse();
-        $dom      = new Dom\Query($response->getContent());
+        $document = new Document($response->getContent());
+
         if ($useXpath) {
-            $dom->registerXpathNamespaces($this->xpathNamespaces);
-            return $dom->queryXpath($path);
+            $document->registerXpathNamespaces($this->xpathNamespaces);
         }
-        return $dom->execute($path);
+
+        $result   = Document\Query::execute($path, $document, $useXpath ? Document\Query::TYPE_XPATH : Document\Query::TYPE_CSS);
+
+        return $result;
     }
 
     /**
